@@ -1,8 +1,10 @@
+import platform
 from random import randint
 import re
-from subprocess import check_output as run, STDOUT
+from subprocess import STDOUT
 
 from autopip.exceptions import MissingError
+from autopip.utils import run
 
 
 def _ensure_cron():
@@ -17,7 +19,10 @@ def _ensure_cron():
         run('ps -ef | grep /usr/sbin/cron | grep -v grep', stderr=STDOUT, shell=True)
 
     except Exception:
-        raise RuntimeError('cron service does not seem to be running. Try starting it: sudo service cron start')
+        if platform.system() == 'Darwin':
+            return  # macOS does not start cron until there is a crontab entry: https://apple.stackexchange.com/a/266836
+
+        raise MissingError(f'cron service does not seem to be running. Try starting it: sudo service cron start')
 
 
 def add(cmd, schedule='? * * * *', cmd_id=None):
@@ -45,7 +50,7 @@ def add(cmd, schedule='? * * * *', cmd_id=None):
     if '?' in schedule:
         schedule = schedule.replace('?', str(randint(0, 59)))
 
-    crontab_cmd = (f'( crontab -l | grep -vi "{cmd_id}"; echo "{schedule} PATH=/usr/local/bin:\$PATH {cmd}" )'
+    crontab_cmd = (rf'( crontab -l | grep -vi "{cmd_id}"; echo "{schedule} PATH=/usr/local/bin:\$PATH {cmd}" )'
                    ' | crontab -')
     run(crontab_cmd, stderr=STDOUT, shell=True)
 
