@@ -2,21 +2,21 @@ from configparser import RawConfigParser
 from collections import defaultdict
 from functools import lru_cache
 import json
-import imp
 from logging import info, error, debug
 import os
 from pathlib import Path, PurePath
 import pkg_resources
 import re
 import shutil
-from subprocess import check_output as run, CalledProcessError, STDOUT
+from subprocess import CalledProcessError, STDOUT
 import sys
 from time import time, sleep
 import urllib.request
 import urllib.error
 
-from autopip.constants import UpdateFreq
 from autopip import crontab, exceptions
+from autopip.constants import UpdateFreq, PYTHON_VERSION
+from autopip.utils import run, sorted_versions
 
 
 class AppsManager:
@@ -35,19 +35,20 @@ class AppsManager:
         # PyPI auth. Tuple of user and password.
         self._index_auth = None
 
-    def install(self, apps, update=None, wait=False):
+    def install(self, apps, update=None, python_version=None, wait=False):
         """
         Install the given apps
 
         :param list[str] apps: List of apps to install
         :param UpdateFreq|None update: How often to update
+        :param str python_version: Python version to run the app
         :param bool wait: Wait for a new version to be published and then install it.
         """
         self._set_index()
 
         autopip_path = shutil.which('autopip')
-        if (self.paths.is_user and sys.stdout.isatty() and not list(self.apps) and autopip_path and
-                autopip_path.startswith(str(self.paths.SYSTEM_SYMLINK_ROOT))):
+        if (self.paths.is_user and sys.stdout.isatty() and not list(self.apps) and autopip_path
+                and autopip_path.startswith(str(self.paths.SYSTEM_SYMLINK_ROOT))):
             info('# Based on permission, this will install to your user home instead of %s',
                  self.paths.SYSTEM_SYMLINK_ROOT)
             info('  To install for everyone, cancel using CTRL+C and then re-run using sudo.')
@@ -57,13 +58,13 @@ class AppsManager:
 
         for name in apps:
             try:
-                if isinstance(name, tuple):  # From app.group_specs() or self.update()
+                if isinstance(name, tuple):  # From app.group_specs()
                     name, update = name
                     if update:
                         update = UpdateFreq.from_name(update)
 
                 app_spec = next(iter(pkg_resources.parse_requirements(name)))
-                app, updated = self._install_app(app_spec, update=update, wait=wait)
+                app, updated = self._install_app(app_spec, update=update, python_version=python_version, wait=wait)
 
                 if updated:
                     printed_wait = False
@@ -88,24 +89,24 @@ class AppsManager:
         if failed_apps:
             raise exceptions.FailedAction()
 
-    def _install_app(self, app_spec, update=None, wait=False):
+    def _install_app(self, app_spec, update=None, python_version=None, wait=False):
         """ Install the given app """
         app = App(app_spec.name, self.paths, debug=self.debug)
         updated = False
 
         # Skip update if install was done within the update frequency when run from cron
-        if (sys.stdout.isatty() or not app.is_installed or
-                update and app.path.stat().st_mtime + update.seconds < time()):
+        if (sys.stdout.isatty() or not app.is_installed or wait
+                or update and app.path.stat().st_mtime + update.seconds < time()):
             if app.is_installed:
                 app.path.touch()
 
             version = self._app_version(app_spec)
 
             if not wait or version != app.current_version:
-                updated = app.install(version, app_spec, update=update)
+                updated = app.install(version, app_spec, update=update, python_version=python_version)
 
         else:
-            debug('App does not need to be updated yet.')
+            debug(f'{app.name} does not need to be updated yet.')
 
         return app, updated
 
@@ -128,9 +129,11 @@ class AppsManager:
 
         except urllib.error.HTTPError as e:
             if e.code == 404:
-                raise NameError(f'App does not exist on {self._index_url}')
+                raise NameError(f'{app_spec.name} does not exist on {self._index_url}')
+            else:
+                raise Exception(f'Failed to read from {pkg_index_url}: {e}')
 
-        version_re = re.compile(app_spec.name + '-(\d+\.\d+\.\d+(?:\.\w+\d+)?)\.')
+        version_re = re.compile(app_spec.name + r'-(\d+\.\d+\.\d+(?:\.\w+\d+)?)\.')
         versions = []
         matched_versions = []
 
@@ -144,35 +147,87 @@ class AppsManager:
 
         if not matched_versions:
             if versions:
-                raise ValueError(f'No app version matching {app_spec} \nAvailable versions: ' + ', '.join(versions))
+                raise ValueError(f'No app version matching {app_spec} \nAvailable versions: '
+                                 + ', '.join(sorted_versions(versions)))
             else:
                 raise ValueError(f'No app version found in {pkg_index_url}')
 
-        version_sep_re = re.compile('[^0-9]+')
-        sorted_versions = sorted(matched_versions, key=lambda v: tuple(map(int, version_sep_re.split(v))))
-
-        return sorted_versions[-1]
+        return sorted_versions(matched_versions)[-1]
 
     def _set_index(self):
         """ Set PyPI url and auth """
         if not self._index_url:
+            for conf_file in ['~/.config/pip/pip.conf', '~/.pip/pip.conf', '/etc/pip.conf']:
+                self._index_url, self._index_auth = self._parse_pip_conf_for_index(conf_file)
+                if self._index_url:
+                    break
+
+            if self._index_url and not self._index_auth:
+                self._index_auth = self._parse_netrc_credential_for(self._index_url)
+
+            if not self._index_url:  # No need to check for login/password for this as everything is public
+                self._index_url = 'https://pypi.org/simple/'
+
+    @staticmethod
+    def _parse_pip_conf_for_index(conf_file):
+        """
+        Parse the given pip.conf file for index-url
+
+        :param str pip_conf_path: Path to pip.conf file
+        :return: Tuple of index_url and another tuple of user and password (or None)
+        """
+        index_url = index_auth = None
+
+        pip_conf = Path(conf_file).expanduser()
+        if pip_conf.exists():
             try:
-                pip_conf = Path('~/.pip/pip.conf').expanduser()
                 parser = RawConfigParser()
                 parser.read(pip_conf)
-                self._index_url = parser.get('global', 'index-url')
+
+                index_url = parser.get('global', 'index-url')
 
                 auth_re = re.compile('//([^:]+)(?::([^@]+))?@')
-                match = auth_re.search(self._index_url)
+                match = auth_re.search(index_url)
                 if match:
-                    self._index_auth = match.groups()
-                    self._index_url = auth_re.sub('//', self._index_url)
+                    index_auth = match.groups()
+                    index_url = auth_re.sub('//', index_url)
 
-                if not self._index_url.endswith('/'):
-                    self._index_url += '/'
+                if not index_url.endswith('/'):
+                    index_url += '/'
 
             except Exception:
-                self._index_url = 'https://pypi.org/simple/'
+                pass
+
+        return index_url, index_auth
+
+    @staticmethod
+    def _parse_netrc_credential_for(index_url, netrc_file='~/.netrc'):
+        """
+        Parse .netrc for login/password for the given index_url
+
+        :param str index_url: Machine URL to find credential for
+        :return: Tuple of login and password or None
+        """
+        netrc = Path(netrc_file).expanduser()
+        if netrc.exists():
+            machine = login = password = None
+            for line in netrc.open():
+                try:
+                    _, machine, _, login, _, password = line.strip().split()
+
+                except Exception:
+                    name, value = line.strip().split()
+                    if name == 'machine':
+                        machine = value
+                    elif name == 'login':
+                        login = value
+                    elif name == 'password':
+                        password = value
+
+                if password:
+                    if machine in index_url:
+                        return (login, password)
+                    machine = login = password = None
 
     @property
     def apps(self):
@@ -239,16 +294,13 @@ class AppsManager:
             info('No apps are installed yet.')
 
             autopip_path = shutil.which('autopip')
-            if (self.paths.is_user and sys.stdout.isatty() and autopip_path and
-                    autopip_path.startswith(str(self.paths.SYSTEM_SYMLINK_ROOT))):
-                info('To see apps installed in %s, re-run using sudo.', self.paths.SYSTEM_SYMLINK_ROOT)
+            if (self.paths.is_user and sys.stdout.isatty() and autopip_path
+                    and autopip_path.startswith(str(self.paths.SYSTEM_SYMLINK_ROOT))):
+                info('To see apps installed in %s, re-run using sudo.', self.paths.SYSTEM_INSTALL_ROOT)
 
     def uninstall(self, apps):
         """ Uninstall apps """
         for name in apps:
-            if name == 'bin':  # Don't try to remove bin (contains symlinks to scripts) from the app dir
-                continue
-
             if name == 'autopip' and len(list(self.apps)) > 1:
                 if apps[-1] == 'autopip':
                     error('! autopip can not be uninstalled until other apps are uninstalled: %s', ' '.join(
@@ -274,7 +326,7 @@ class AppsManager:
             try:
                 crontab.remove('autopip')
             except Exception as e:
-                debug(e)
+                debug('Could not remove crontab for autopip: %s', e)
 
     def update(self, apps=None, wait=False):
         """
@@ -291,7 +343,7 @@ class AppsManager:
                 settings = app.settings()
                 if settings.get('update'):
                     app_specs.append((settings['app_spec'], settings['update']))
-                elif sys.stdout.isatty():
+                elif sys.stdout.isatty() or wait:
                     app_specs.append((settings.get('app_spec', app.name), None))
 
             if app_specs:
@@ -301,7 +353,7 @@ class AppsManager:
                 try:
                     crontab.remove('autopip')
                 except Exception as e:
-                    debug(e)
+                    debug('Could not remove crontab for autopip: %s', e)
 
         elif list(self.apps):
             info('No apps found matching: %s', ', '.join(apps))
@@ -336,6 +388,9 @@ class App:
         # Unique crontab name to to easily add and remove from crontab
         self._crontab_id = rf'autopip install "{self.name}[^a-z]*"'
 
+    def __repr__(self):
+        return f"App('{self.name}')"
+
     @property
     def is_installed(self):
         """ Is the app installed? """
@@ -352,19 +407,30 @@ class App:
         if self.current_path:
             return self.current_path.resolve().name
 
-    def install(self, version, app_spec, update=None):
+    def install(self, version, app_spec, update=None, python_version=None):
         """
         Install the version of the app if it is not already installed
 
         :param str version: Version of the app to install
         :param pkg_resources.Requirement app_spec: App version requirement from user
         :param UpdateFreq|None update: How often to update. Choose from hourly, daily, weekly, monthly
+        :param str python_version: Python version to run app
         :return: True if install or update happened, otherwise False when nothing happened (already installed / non-tty)
         """
         version_path = self.path / version
         prev_version_path = self.current_path and self.current_path.resolve()
         important_paths = [version_path, prev_version_path, self._current_symlink]
 
+        if self.settings():
+            if not python_version:
+                python_version = self.settings().get('python_version')
+
+            if not update:
+                update = self.settings().get('update') and UpdateFreq.from_name(self.settings()['update'])
+
+        if not python_version:
+            python_version = PYTHON_VERSION
+
         if version_path.exists():
             if self.current_version == version:
                 # Skip printing / ensuring symlinks / cronjob when running from cron
@@ -380,29 +446,57 @@ class App:
                 info(f'{self.name} {version} was previously installed and will be set as the current version')
 
         else:
+            if not shutil.which('python' + python_version):
+                error(f'! python{python_version} does not exist. '
+                      'Please install it first, or ensure its path is in PATH.')
+                sys.exit(1)
+
+            if python_version.startswith('2'):
+                venv = f'virtualenv --python=python{python_version}'
+            else:
+                venv = f'python{python_version} -m venv'
+
             old_venv_dir = None
             old_path = None
+            no_compile = '--no-compile ' if os.getuid() else ''
+
+            info(f'Installing {self.name} to {version_path}')
+
+            os.environ.pop('PYTHONPATH', None)
+            if 'VIRTUAL_ENV' in os.environ:
+                old_venv_dir = os.environ.pop('VIRTUAL_ENV')
+                old_path = os.environ['PATH']
+                os.environ['PATH'] = os.pathsep.join([p for p in os.environ['PATH'].split(os.pathsep)
+                                                      if os.path.exists(p) and not p.startswith(old_venv_dir)])
 
             try:
-                info(f'Installing {self.name} to {version_path}')
-                if 'VIRTUAL_ENV' in os.environ:
-                    old_venv_dir = os.environ.pop('VIRTUAL_ENV')
-                    old_path = os.environ['PATH']
-                    os.environ['PATH'] = os.pathsep.join([p for p in os.environ['PATH'].split(os.pathsep)
-                                                          if os.path.exists(p) and not p.startswith(old_venv_dir)])
-
                 run(f"""set -e
-                    python3.6 -m venv {version_path}
-                    source {version_path / 'bin/activate'}
-                    pip install --upgrade pip
-                    pip install {self.name}=={version}
+                    {venv} {version_path}
+                    source {version_path / 'bin' / 'activate'}
+                    pip install --upgrade pip wheel
+                    pip install {no_compile}{self.name}=={version}
                     """, executable='/bin/bash', stderr=STDOUT, shell=True)
 
             except BaseException as e:
                 shutil.rmtree(version_path, ignore_errors=True)
 
-                if isinstance(e, CalledProcessError) and e.output:
-                    info(re.sub(r'(https?://)[^/]+:[^/]+@', r'\1<xxx>:<xxx>@', e.output.decode('utf-8')))
+                if isinstance(e, CalledProcessError):
+                    if e.output:
+                        output = e.output.decode('utf-8')
+
+                        if not python_version.startswith('2'):
+                            python2 = shutil.which('python2') or shutil.which('python2.7')
+                            if python2:
+                                py2_version = Path(python2).name.lstrip('python')
+                                if 'is a builtin module since Python 3' in output:
+                                    info(f'Failed to install using Python {python_version} venv, '
+                                         f'let\'s try using Python {py2_version} virtualenv.')
+                                    return self.install(version, app_spec, update=update, python_version=py2_version)
+
+                        info(re.sub(r'(https?://)[^/]+:[^/]+@', r'\1<xxx>:<xxx>@', output))
+
+                    error(f'! Failed to install using Python {python_version}.'
+                          ' If this app requires a different Python version, please specify it using --python option.')
 
                 raise
 
@@ -411,6 +505,16 @@ class App:
                     os.environ['VIRTUAL_ENV'] = old_venv_dir
                     os.environ['PATH'] = old_path
 
+            try:
+                shutil.rmtree(version_path / 'share' / 'python-wheels', ignore_errors=True)
+                run(f"""set -e
+                    source {version_path / 'bin' / 'activate'}
+                    pip uninstall --yes pip
+                    """, executable='/bin/bash', stderr=STDOUT, shell=True)
+
+            except Exception as e:
+                debug('Could not remove unnecessary packages/files: %s', e)
+
         # Update current symlink
         if not self.current_path or self.current_path.resolve() != version_path:
             atomic_symlink = self.path / f'atomic_symlink_for_{self.name}'
@@ -431,7 +535,7 @@ class App:
                 '  If you are the app owner, make sure to setup entry_points in setup.py.\n'
                 '  See http://setuptools.readthedocs.io/en/latest/setuptools.html#automatic-script-creation')
 
-        self.settings(app_spec=str(app_spec))
+        self.settings(app_spec=str(app_spec), python_version=python_version)
 
         # Install cronjobs
         if 'update' not in sys.argv:
@@ -445,7 +549,7 @@ class App:
                     try:
                         crontab.remove(self._crontab_id)
                     except exceptions.MissingError as e:
-                        debug(e)
+                        debug('Could not remove crontab for %s: %s', self._crontab_id, e)
 
             elif update:
                 try:
@@ -456,7 +560,7 @@ class App:
 
                     # Migrate old crontabs
                     try:
-                        old_crons = [c for c in crontab.list().decode().split('\n') if c and 'autopip update' not in c]
+                        old_crons = [c for c in crontab.list().split('\n') if c and 'autopip update' not in c]
                         if old_crons:
                             cron_re = re.compile('autopip install "(.+)"')
                             for cron in old_crons:
@@ -469,7 +573,7 @@ class App:
                             crontab.remove('autopip')
 
                     except Exception as e:
-                        debug(e)
+                        debug('Could not migrate old crontabs: %s', e)
 
                     crontab.add(f'{autopip_path} update '
                                 f'2>&1 >> {self.paths.log_root / "cron.log"}', cmd_id='autopip update')
@@ -520,6 +624,13 @@ class App:
         if not printed_updating and sys.stdout.isatty() and current_scripts and 'update' not in sys.argv:
             info('Scripts are in {}: {}'.format(self.paths.symlink_root, ', '.join(sorted(current_scripts))))
 
+        # Remove pyc for non-root installs for all versions, not just current.
+        if os.getuid():
+            try:
+                run(f'find {self.path} -name *.pyc | xargs rm', executable='/bin/bash', stderr=STDOUT, shell=True)
+            except Exception as e:
+                debug('Could not remove *.pyc files: %s', e)
+
         return True
 
     def settings(self, **new_settings):
@@ -543,95 +654,50 @@ class App:
 
     def scripts(self, path=None):
         """ Set of scripts for the given app path (defaults to current). """
-        dist = self._pkg_distribution(path=path)
-
-        if dist:
-            console_scripts = dist.get_entry_map('console_scripts')
-
-            if console_scripts:
-                return set(console_scripts.keys())
-
-            try:
-                records = dist.get_metadata('RECORD')
-            except Exception:
-                records = dist.get_metadata('installed-files.txt')
-
-            if records:
-                scripts = set()
-                bin_re = re.compile('../bin/([^,]+),?')
-
-                for line in records.split('\n'):
-                    match = bin_re.search(line)
-
-                    if match:
-                        scripts.add(match.group(1))
-
-                return scripts
-
-        return set()
+        dist = self._pkg_info(path=path)
+        return dist and set(dist['scripts']) or set()
 
     def group_specs(self, path=None, name_only=False):
         """ List of app specs from this app's "autopip" entry points for the given app path (defaults to current)"""
         app_specs = []
-        dist = self._pkg_distribution(path=path)
+        dist = self._pkg_info(path=path)
 
         if dist:
-            for app, spec in dist.get_entry_map('autopip').items():
+            for app, version, update in dist.get('group_specs', []):
                 if name_only:
                     app_specs.append(app)
 
-                elif spec.module_name == 'latest':
-                    app_specs.append((app, UpdateFreq.DEFAULT.name.lower()))
+                elif version == 'latest':
+                    app_specs.append((app, update or UpdateFreq.DEFAULT.name.lower()))
 
-                else:
-                    if len(spec.module_name.split('.')) < 3 or spec.extras:     # Wildcard with auto-update
-                        app_specs.append((f'{app}=={spec.module_name}.*',
-                                         next(iter(spec.extras), UpdateFreq.DEFAULT.name.lower())))
-                    else:
-                        app_specs.append((f'{app}=={spec.module_name}', None))  # Specific version without auto-update
+                elif len(version.split('.')) < 3:
+                    app_specs.append((f'{app}=={version}.*', update or UpdateFreq.DEFAULT.name.lower()))
+
+                else:  # Specific version without auto-update
+                    app_specs.append((f'{app}=={version}', None))
 
         return app_specs
 
     @lru_cache()
-    def _pkg_distribution(self, path=None):
-        """ Get pkg_resources.Distribution() for the given path (defaults to current path) """
+    def _pkg_info(self, path=None):
+        """ Get scripts and entry points from the app """
         if not path:
             if not self.current_path:
                 return
 
             path = self.current_path
 
+        inspect_py = Path(__file__).parent / 'inspect_app.py'
+
         try:
-            return pkg_resources.get_distribution(self.name)
+            info = run(f"""set -e
+                source {path / 'bin/activate'}
+                python {inspect_py} {self.name}
+                """, executable='/bin/bash', stderr=STDOUT, shell=True)
+            return json.loads(info)
 
-        except pkg_resources.DistributionNotFound:
-            activated = False
-
-            try:
-                activate_this_file = path / 'bin' / 'activate_this.py'
-
-                if not activate_this_file.exists():
-                    shutil.copyfile(Path(__file__).parent / 'embedded' / 'activate_this.py', activate_this_file)
-
-                old_os_path = os.environ.get('PATH', '')
-                old_sys_path = list(sys.path)
-                old_prefix = sys.prefix
-
-                exec(open(activate_this_file).read(), dict(__file__=activate_this_file))
-                activated = True
-
-                imp.reload(pkg_resources)
-                return pkg_resources.get_distribution(self.name)
-
-            except Exception as e:
-                error('! Can not get package distribution info because: %s', e)
-
-            finally:
-                if activated:
-                    os.environ['PATH'] = old_os_path
-                    sys.path = old_sys_path
-                    sys.prefix = old_prefix
-                    imp.reload(pkg_resources)
+        except Exception as e:
+            debug('! Can not get package distribution info because: %s', e)
 
     def uninstall(self):
         """ Uninstall app """
@@ -640,12 +706,12 @@ class App:
         try:
             crontab.remove(self._crontab_id)
         except exceptions.MissingError as e:
-            debug(e)
+            debug('Could not remove crontab for %s: %s', self._crontab_id, e)
 
         for script in self.scripts():
             script_symlink = self.paths.symlink_root / script
-            if ((script_symlink.exists() or script_symlink.is_symlink()) and
-                    str(script_symlink.resolve()).startswith(str(self.path))):
+            if ((script_symlink.exists() or script_symlink.is_symlink())
+                    and str(script_symlink.resolve()).startswith(str(self.path))):
                 script_symlink.unlink()
 
         shutil.rmtree(self.path)
@@ -669,9 +735,9 @@ class AppsPath:
     LOCAL_LOG_ROOT = _LOCAL_BASE / 'var' / 'log' / 'autopip'
 
     # User install paths
-    USER_INSTALL_ROOT = Path('~/.apps').expanduser()
-    USER_SYMLINK_ROOT = USER_INSTALL_ROOT / 'bin'
-    USER_LOG_ROOT = USER_INSTALL_ROOT / 'log'
+    USER_INSTALL_ROOT = Path.home() / '.apps'
+    USER_SYMLINK_ROOT = Path.home() / 'bin'
+    USER_LOG_ROOT = USER_INSTALL_ROOT / '.log'
 
     def __init__(self):
         #: Root to install apps. This will be set at runtime based on permission by :meth:`_set_roots`
@@ -708,16 +774,17 @@ class AppsPath:
 
         # Check local
         if system_reasons:
-            if not (os.access(self.LOCAL_INSTALL_ROOT.parent, os.W_OK) or
-                    not self.LOCAL_INSTALL_ROOT.parent.exists() and
-                    os.access(self.LOCAL_INSTALL_ROOT.parent.parent, os.W_OK)):
+            if not (os.access(self.LOCAL_INSTALL_ROOT.parent, os.W_OK)
+                    or not self.LOCAL_INSTALL_ROOT.parent.exists()
+                    and os.access(self.LOCAL_INSTALL_ROOT.parent.parent, os.W_OK)):
                 local_reasons.append(f'No permission to write to {self.LOCAL_INSTALL_ROOT.parent}')
 
             if not os.access(self.LOCAL_SYMLINK_ROOT, os.W_OK):
                 local_reasons.append(f'No permission to write to {self.LOCAL_SYMLINK_ROOT}')
 
-            if not (os.access(self.LOCAL_LOG_ROOT.parent, os.W_OK) or
-                    not self.LOCAL_LOG_ROOT.parent.exists() and os.access(self.LOCAL_LOG_ROOT.parent.parent, os.W_OK)):
+            if not (os.access(self.LOCAL_LOG_ROOT.parent, os.W_OK)
+                    or not self.LOCAL_LOG_ROOT.parent.exists()
+                    and os.access(self.LOCAL_LOG_ROOT.parent.parent, os.W_OK)):
                 local_reasons.append(f'No permission to write to {self.LOCAL_LOG_ROOT.parent}')
 
             if local_reasons:
@@ -746,6 +813,6 @@ class AppsPath:
     def covers(self, path):
         """ True if the given path belongs to autopip """
         path = path.resolve() if isinstance(path, PurePath) else path
-        return (str(path).startswith(str(self.SYSTEM_INSTALL_ROOT)) or
-                str(path).startswith(str(self.LOCAL_INSTALL_ROOT)) or
-                str(path).startswith(str(self.USER_INSTALL_ROOT)))
+        return (str(path).startswith(str(self.SYSTEM_INSTALL_ROOT))
+                or str(path).startswith(str(self.LOCAL_INSTALL_ROOT))
+                or str(path).startswith(str(self.USER_INSTALL_ROOT)))
